C ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ SUBROUTINE BESK(X,N,BK,IER)
C usage:  CALL BESK(X,N,BK,IER)
C
C           COMPUTE THE K BESSEL FUNCTION FOR A GIVEN ARGUMENT AND ORDER
C
C        DESCRIPTION OF PARAMETERS
C           X  -THE ARGUMENT OF THE K BESSEL FUNCTION DESIRED
C           N  -THE ORDER OF THE K BESSEL FUNCTION DESIRED
C           BK -THE RESULTANT K BESSEL FUNCTION
C           IER-RESULTANT ERROR CODE WHERE
C              IER=0  NO ERROR
C              IER=1  N IS NEGATIVE
C              IER=2  X IS ZERO OR NEGATIVE
C              the following are machine dependant:
C              IER=3  IF X.GT.101, BOTH K0(X) AND K1(X) ARE ZERO
C                     AND RECURRENCE NO LONGER MEANINGFUL
C              IER=4  BK .GT. 10**36
C
C        REMARKS
C           N MUST BE GREATER THAN OR EQUAL TO ZERO
C           THERE ARE NO RESTRICTIONS ON THE ORDER, BUT FOR A GIVEN X,
C           KN(X) TENDS TO INFINITY AS N TENDS TO INFINITY AND A LARGE
C           ORDER MAY CAUSE OVERFLOW.
C           FOR ARGUMENTS LESS THAN OR EQUAL TO 10 , THE ORDER
C           REQUESTED SHOULD BE LESS THAN OR EQUAL TO 60.
C           On the SUN (IEEE FP format), for N=1, the
C           result becomes inaccurate for X > 85, and becomes
C           zero for X = 102. Use DBESK if this is unacceptable
C           in DBESK the range may be as large as X=700
C
C        METHOD
C           COMPUTES ZERO ORDER AND FIRST ORDER BESSEL FUNCTIONS USING
C           SERIES APPROXIMATIONS AND THEN COMPUTES N TH ORDER FUNCTION
C           USING RECURRENCE RELATION.
C           RECURRENCE RELATION AND POLYNOMIAL APPROXIMATION TECHNIQUE
C           AS DESCRIBED BY A.J.M.HITCHCOCK,'POLYNOMIAL APPROXIMATIONS
C           TO BESSEL FUNCTIONS OF ORDER ZERO AND ONE AND TO RELATED
C           FUNCTIONS', M.T.A.C., V.11,1957,PP.86-88, AND G.N. WATSON,
C           'A TREATISE ON THE THEORY OF BESSEL FUNCTIONS', CAMBRIDGE
C           UNIVERSITY PRESS, 1958, P. 62
C THE ORIGINAL FORM OF THIS SUBROUTINE WAS TAKEN FROM
C "SCIENTIFIC SUBROUTINE PACKAGE (SSP)"
C INTERCHANGE LIBRARY (PERKIN-ELMER USERS GROUP), IUG-320.
C
C     ..................................................................
C
      SUBROUTINE BESK(X,N,BK,IER)
      DIMENSION T(12)
      BK=.0
      IER=0
      G0=9.9E36
      GJ=9.9E36
      IF(N.LT.0)IER=1
      IF(X.LE.0.0)IER=2
      IF(X.GT.101.0)IER=3
      IF(IER.NE.0)RETURN
      IF(X.LE.1.0)GOTO 36
      A=EXP(-X)
      B=1./X
      C=SQRT(B)
      T(1)=B
      DO 26 L=2,12
   26 T(L)=T(L-1)*B
      IF(N.EQ.1)GOTO 29
C
C     COMPUTE KO USING POLYNOMIAL APPROXIMATION
C
      G0=A*(1.2533141-.1566642*T(1)+.08811128*T(2)-.09139095*T(3)
     2+.1344596*T(4)-.2299850*T(5)+.3792410*T(6)-.5247277*T(7)
     3+.5575368*T(8)-.4262633*T(9)+.2184518*T(10)-.06680977*T(11)
     4+.009189383*T(12))*C
      IF(N.NE.0)GOTO 29
      BK=G0
      RETURN
C
C     COMPUTE K1 USING POLYNOMIAL APPROXIMATION
C
   29 G1=A*(1.2533141+.4699927*T(1)-.1468583*T(2)+.1280427*T(3)
     2-.1736432*T(4)+.2847618*T(5)-.4594342*T(6)+.6283381*T(7)
     3-.6632295*T(8)+.5050239*T(9)-.2581304*T(10)+.07880001*T(11)
     4-.01082418*T(12))*C
      IF(N.NE.1)GOTO 31
      BK=G1
      RETURN
C
C     FROM KO,K1 COMPUTE KN USING RECURRENCE RELATION
C
   31 DO 35 J=2,N
      GJ=2.*(FLOAT(J)-1.)*G1/X+G0
      IF(GJ.GT.1.0E36 .OR. G0.GT.1.0E36)THEN
          IER=4
          GOTO 34
      ENDIF
      G0=G1
   35 G1=GJ
   34 BK=GJ
      RETURN
   36 B=X/2.
      A=.5772157+ALOG(B)
      C=B*B
      IF(N.EQ.1)GOTO 43
C
C     COMPUTE KO USING SERIES EXPANSION
C
      G0=-A
      X2J=1.
      FACT=1.
      HJ=.0
C FOLLOWING LOOP HAD A FIXED NUMBER (6) OF ITERATIONS.
C MODIFIED TO AVOID UNDERFLOW FOR VERY SMALL ARGUMENTS. 10/11/81.
C A SIMILAR CHANGE WAS MADE IN THE CALCULATION OF K1.
      QJ=0.0
  40  QJ=QJ+1.0
      RJ=1.0/QJ
      X2J=X2J*C
      FACT=FACT*RJ*RJ
      HJ=HJ+RJ
      G0=G0+X2J*FACT*(HJ-A)
      IF(X2J.LT.1.0E-4)GOTO 41
      GO TO 40
   41 IF(N.NE.0)GOTO 43
      BK=G0
      RETURN
C
C     COMPUTE K1 USING SERIES EXPANSION
C
   43 X2J=B
      FACT=1.
      HJ=1.
      G1=1./X+X2J*(.5+A-HJ)
      FLJ=2.0
   50 X2J=X2J*C
      RJ=1./FLJ
      FACT=FACT*RJ*RJ
      HJ=HJ+RJ
      G1=G1+X2J*FACT*(.5+(A-HJ)*FLJ)
      IF(X2J.LT.1.0E-4)GOTO 51
      FLJ=FLJ+1.0
      GOTO 50
   51 IF(N.NE.1)GOTO 31
      BK=G1
      END
